package com.flipperdevices.core.ui.errors.impl.api

import androidx.compose.foundation.lazy.LazyListScope
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import com.flipperdevices.core.di.AppGraph
import com.flipperdevices.core.ui.errors.impl.ComposableThrowableErrorInternal
import com.flipperdevices.deeplink.model.Deeplink
import com.flipperdevices.deeplink.model.DeeplinkBottomBarTab
import com.flipperdevices.faphub.errors.api.FapErrorSize
import com.flipperdevices.faphub.errors.api.FapHubComposableErrorsRenderer
import com.flipperdevices.faphub.errors.api.throwable.FapHubError
import com.flipperdevices.rootscreen.api.LocalDeeplinkHandler
import com.squareup.anvil.annotations.ContributesBinding
import javax.inject.Inject

@ContributesBinding(AppGraph::class, FapHubComposableErrorsRenderer::class)
class FapHubComposableErrorsRendererImpl @Inject constructor() : FapHubComposableErrorsRenderer {

    override fun LazyListScope.ComposableThrowableErrorListItem(
        throwable: FapHubError,
        onRetry: () -> Unit,
        modifier: Modifier,
        fapErrorSize: FapErrorSize
    ) {
        item {
            var throwableModifier = modifier

            if (fapErrorSize == FapErrorSize.FULLSCREEN) {
                throwableModifier = throwableModifier.fillParentMaxHeight()
            }

            ComposableThrowableError(
                throwable = throwable,
                onRetry = onRetry,
                modifier = throwableModifier,
                fapErrorSize = fapErrorSize
            )
        }
    }

    @Composable
    override fun ComposableThrowableError(
        throwable: FapHubError,
        onRetry: () -> Unit,
        modifier: Modifier,
        fapErrorSize: FapErrorSize
    ) {
        val deeplinkHandler = LocalDeeplinkHandler.current
        ComposableThrowableErrorInternal(
            throwable = throwable,
            onRetry = onRetry,
            modifier = modifier,
            fapErrorSize = fapErrorSize,
            onOpenDeviceScreen = {
                deeplinkHandler.handleDeeplink(Deeplink.BottomBar.OpenTab(DeeplinkBottomBarTab.DEVICE))
            }
        )
    }
}
